/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.states.minfo;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.CallMeta;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.BedQualityAccess;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.DataFacet;
import org.dive4elements.river.artifacts.model.DateRange;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.minfo.BedDensityFacet;
import org.dive4elements.river.artifacts.model.minfo.BedDiameterDataFacet;
import org.dive4elements.river.artifacts.model.minfo.BedDiameterFacet;
import org.dive4elements.river.artifacts.model.minfo.BedDiameterResult;
import org.dive4elements.river.artifacts.model.minfo.BedParametersResult;
import org.dive4elements.river.artifacts.model.minfo.BedPorosityFacet;
import org.dive4elements.river.artifacts.model.minfo.BedQualityCalculation;
import org.dive4elements.river.artifacts.model.minfo.BedQualityDiameterResult;
import org.dive4elements.river.artifacts.model.minfo.BedQualityResult;
import org.dive4elements.river.artifacts.model.minfo.BedloadDiameterFacet;
import org.dive4elements.river.artifacts.model.minfo.BedloadDiameterResult;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.states.DefaultState;


public class BedQualityState extends DefaultState implements FacetTypes {

    private static final long serialVersionUID = 1L;

    private static final Logger logger = Logger
        .getLogger(BedQualityState.class);

    public static final String I18N_TOPLAYER = "bedquality.toplayer";
    public static final String I18N_SUBLAYER = "bedquality.sublayer";

    public static final String I18N_FACET_BED_POROSITY_TOPLAYER = "facet.bedquality.bed.porosity.toplayer";
    public static final String I18N_FACET_BED_POROSITY_SUBLAYER = "facet.bedquality.bed.porosity.sublayer";
    public static final String I18N_FACET_BED_DENSITY_TOPLAYER = "facet.bedquality.bed.density.toplayer";
    public static final String I18N_FACET_BED_DENSITY_SUBLAYER = "facet.bedquality.bed.density.sublayer";
    public static final String I18N_FACET_BED_DIAMETER_TOPLAYER = "facet.bedquality.bed.diameter.toplayer";
    public static final String I18N_FACET_BED_DIAMETER_SUBLAYER = "facet.bedquality.bed.diameter.sublayer";
    public static final String I18N_FACET_BED_DIAMETER_DATA_TOPLAYER = "facet.bedquality.bed.diameter.data.toplayer";
    public static final String I18N_FACET_BED_DIAMETER_DATA_SUBLAYER = "facet.bedquality.bed.diameter.data.sublayer";
    public static final String I18N_FACET_BEDLOAD_DIAMETER_DATA = "facet.bedquality.bedload.diameter.data";
    public static final String I18N_FACET_BEDLOAD_DIAMETER = "facet.bedquality.bedload.diameter";

    @Override
    public Object computeAdvance(D4EArtifact artifact, String hash,
        CallContext context, List<Facet> facets, Object old) {
        logger.debug("BedQualityState.computeAdvance");

        List<Facet> newFacets = new ArrayList<Facet>();

        BedQualityAccess access = new BedQualityAccess(artifact, context);

        CalculationResult res = old instanceof CalculationResult ? (CalculationResult) old
            : new BedQualityCalculation().calculate(access);

        if (facets == null || res == null) {
            return res;
        }

        BedQualityResult[] results = (BedQualityResult[]) res.getData();

        if (results == null || results.length == 0) {
            logger.warn("Calculation computed no results!");
            return res;
        }

        generateFacets(context, newFacets, results, getID(), hash);
        logger.debug("Created " + newFacets.size() + " new Facets.");
        generateDataFacets(context, newFacets, access, getID(), hash);
        facets.addAll(newFacets);

        return res;
    }

    private void generateDataFacets(
        CallContext context,
        List<Facet> newFacets,
        BedQualityAccess access,
        String stateId,
        String hash) {
        List<String> diameters = access.getBedDiameter();
        List<DateRange> ranges = access.getDateRanges();
        for (int i = 0; i < ranges.size(); i++) {
            DateRange range = ranges.get(i);
            for (String diameter: diameters) {
                int ndxTop = generateIndex(diameter, true);
                int ndxSub = generateIndex(diameter, false);
                String toplayer =
                    Resources.getMsg(
                        context.getMeta(), I18N_TOPLAYER, I18N_TOPLAYER);
                String sublayer =
                    Resources.getMsg(
                        context.getMeta(), I18N_SUBLAYER, I18N_SUBLAYER);
                //toplayer
                newFacets.add(new BedDiameterDataFacet(
                    ndxTop,
                    BED_DIAMETER_DATA_TOP,
                    Resources.getMsg(
                        context.getMeta(),
                        I18N_FACET_BED_DIAMETER_DATA_TOPLAYER,
                        I18N_FACET_BED_DIAMETER_DATA_TOPLAYER,
                        new Object[] { diameter.toUpperCase(),
                            range.getFrom(), range.getTo(), toplayer}),
                    ComputeType.ADVANCE,
                    stateId,
                    hash));
                //sublayer
                newFacets.add(new BedDiameterDataFacet(
                    ndxSub,
                    BED_DIAMETER_DATA_SUB,
                    Resources.getMsg(
                        context.getMeta(),
                        I18N_FACET_BED_DIAMETER_DATA_TOPLAYER,
                        I18N_FACET_BED_DIAMETER_DATA_TOPLAYER,
                        new Object[] { diameter.toUpperCase(),
                            range.getFrom(), range.getTo(), sublayer}),
                    ComputeType.ADVANCE,
                    stateId,
                    hash));
            }
        }
    }

    private int generateIndex(String diameter, boolean b) {
        int d = 0;
        if(diameter.equals("d10")) {
            d = 1;
        }
        else if (diameter.equals("d16")) {
            d = 2;
        }
        else if (diameter.equals("d20")) {
            d = 3;
        }
        else if (diameter.equals("d25")) {
            d = 4;
        }
        else if (diameter.equals("d30")) {
            d = 5;
        }
        else if (diameter.equals("d40")) {
            d = 6;
        }
        else if (diameter.equals("d50")) {
            d = 7;
        }
        else if (diameter.equals("d60")) {
            d = 8;
        }
        else if (diameter.equals("d70")) {
            d = 9;
        }
        else if (diameter.equals("d75")) {
            d = 10;
        }
        else if (diameter.equals("d80")) {
            d = 11;
        }
        else if (diameter.equals("d84")) {
            d = 12;
        }
        else if (diameter.equals("d90")) {
            d = 13;
        }
        else if (diameter.equals("dmin")) {
            d = 14;
        }
        else if (diameter.equals("dmax")) {
            d = 15;
        }
        else if (diameter.equals("dm")) {
            d = 16;
        }
        int ndx = d;
        ndx = ndx << 1;
        if (b) {
            ndx += 1;
        }
        ndx = ndx << 3;
        return ndx;
    }

    protected void generateFacets(CallContext context, List<Facet> newFacets,
        BedQualityResult[] results, String stateId, String hash) {
        logger.debug("BedQualityState.generateFacets");

        CallMeta meta = context.getMeta();

        newFacets.add(new DataFacet(CSV, "CSV data", ComputeType.ADVANCE, hash, id));
        for (int idx = 0; idx < results.length; idx++) {
            BedQualityResult result = results[idx];
            DateRange range = result.getDateRange();
            BedDiameterResult[] bedDiameter = result.getBedResults();
            for (int j = 0; j < bedDiameter.length; j++) {
                newFacets.add(new BedDiameterFacet((idx << 8) + j,
                    BED_QUALITY_BED_DIAMETER_TOPLAYER,
                    createDiameterTopLayerDescription(
                        meta,
                        bedDiameter[j],
                        range),
                    ComputeType.ADVANCE, stateId, hash));

                newFacets.add(new BedDiameterFacet((idx << 8) +j,
                    BED_QUALITY_BED_DIAMETER_SUBLAYER,
                    createDiameterSubLayerDescription(
                        meta,
                        bedDiameter[j],
                        range),
                    ComputeType.ADVANCE, stateId, hash));
            }
            BedloadDiameterResult[] bedloadDiameter = result.getBedloadResults();
            for (int j = 0;  j < bedloadDiameter.length; j++) {
                newFacets.add(new BedloadDiameterFacet(
                    (idx << 8) + j,
                    BED_QUALITY_BEDLOAD_DIAMETER,
                    createDiameterDescription(
                        meta, bedloadDiameter[j]),
                    ComputeType.ADVANCE,
                    stateId,
                    hash));

            }
            BedParametersResult[] bedParameters = result.getParameters();
            for (int j = 0; j < bedParameters.length; j++) {
                newFacets.add(new BedPorosityFacet((idx << 8) + j,
                    BED_QUALITY_POROSITY_TOPLAYER,
                    createPorosityTopLayerDescription(
                        meta,
                        bedParameters[j],
                        range),
                    ComputeType.ADVANCE, stateId, hash));

                newFacets.add(new BedPorosityFacet((idx << 8) + j,
                    BED_QUALITY_POROSITY_SUBLAYER,
                    createPorositySubLayerDescription(
                        meta,
                        bedParameters[j],
                        range),
                    ComputeType.ADVANCE, stateId, hash));

                newFacets.add(new BedDensityFacet((idx << 8) + j,
                    BED_QUALITY_SEDIMENT_DENSITY_TOPLAYER,
                    createDensityTopLayerDescription(
                        meta,
                        bedParameters[j],
                        range),
                    ComputeType.ADVANCE, stateId, hash));

                newFacets.add(new BedDensityFacet((idx << 8) + j,
                    BED_QUALITY_SEDIMENT_DENSITY_SUBLAYER,
                    createDensitySubLayerDescription(
                        meta,
                        bedParameters[j],
                        range),
                    ComputeType.ADVANCE, stateId, hash));
            }
        }
    }

    protected String createPorosityTopLayerDescription(CallMeta meta,
        BedParametersResult result, DateRange range) {
        Date from = range != null ? range.getFrom() : new Date();
        Date to = range != null ? range.getTo() : new Date();

        String toplayer = Resources.getMsg(meta, I18N_TOPLAYER, I18N_TOPLAYER);
        return Resources.getMsg(meta, I18N_FACET_BED_POROSITY_TOPLAYER,
            I18N_FACET_BED_POROSITY_TOPLAYER, new Object[] { from, to, toplayer });
    }

    protected String createPorositySubLayerDescription(CallMeta meta,
        BedParametersResult result, DateRange range) {
        Date from = range != null ? range.getFrom() : new Date();
        Date to = range != null ? range.getTo() : new Date();

        String sublayer = Resources.getMsg(meta, I18N_SUBLAYER, I18N_SUBLAYER);
        return Resources.getMsg(meta, I18N_FACET_BED_POROSITY_SUBLAYER,
            I18N_FACET_BED_POROSITY_SUBLAYER, new Object[] { from, to, sublayer });
    }

    protected String createDensityTopLayerDescription(CallMeta meta,
        BedParametersResult result, DateRange range) {
        Date from = range != null ? range.getFrom() : new Date();
        Date to = range != null ? range.getTo() : new Date();

        String toplayer = Resources.getMsg(meta, I18N_TOPLAYER, I18N_TOPLAYER);
        return Resources.getMsg(meta, I18N_FACET_BED_DENSITY_TOPLAYER,
            I18N_FACET_BED_DENSITY_TOPLAYER, new Object[] { from, to, toplayer });
    }

    protected String createDensitySubLayerDescription(CallMeta meta,
        BedParametersResult result, DateRange range) {
        Date from = range != null ? range.getFrom() : new Date();
        Date to = range != null ? range.getTo() : new Date();

        String sublayer = Resources.getMsg(meta, I18N_SUBLAYER, I18N_SUBLAYER);
        return Resources.getMsg(meta, I18N_FACET_BED_DENSITY_SUBLAYER,
            I18N_FACET_BED_DENSITY_SUBLAYER, new Object[] { from, to, sublayer });
    }

    protected String createDiameterTopLayerDescription(CallMeta meta,
        BedDiameterResult result, DateRange range) {
        Date from = range != null ? range.getFrom() : new Date();
        Date to = range != null ? range.getTo() : new Date();

        String toplayer = Resources.getMsg(meta, I18N_TOPLAYER, I18N_TOPLAYER);

        return Resources.getMsg(meta, I18N_FACET_BED_DIAMETER_TOPLAYER,
            I18N_FACET_BED_DIAMETER_TOPLAYER, new Object[] { result.getType(),
                from, to, toplayer });
    }

    protected String createDiameterSubLayerDescription(CallMeta meta,
        BedDiameterResult result, DateRange range) {
        Date from = range != null ? range.getFrom() : new Date();
        Date to = range != null ? range.getTo() : new Date();

        String sublayer = Resources.getMsg(meta, I18N_SUBLAYER, I18N_SUBLAYER);
        return Resources.getMsg(meta, I18N_FACET_BED_DIAMETER_SUBLAYER,
            I18N_FACET_BED_DIAMETER_SUBLAYER, new Object[] { result.getType(),
                from, to, sublayer });
    }

    protected String createDiameterDescription(CallMeta meta,
        BedQualityDiameterResult result) {
        return Resources.getMsg(meta, I18N_FACET_BEDLOAD_DIAMETER,
            I18N_FACET_BEDLOAD_DIAMETER, new Object[] { result.getType() });
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
